<?php
/**
 * Created by PhpStorm.
 * User: USER
 * Date: 6/7/18
 * Time: 5:52 PM
 */

namespace Drupal\drupal_helper;

use Drupal\file\Entity\File;
use Drupal\Core\Url;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Database\Database;
use Drupal\image\Entity\ImageStyle;
class DrupalCommonHelper
{
    public function taxonomy_load_by_tid($tid, $type = "teaser")
    {
        if(is_numeric($tid)){
            $taxonomy_term = \Drupal::entityTypeManager()
                ->getStorage('taxonomy_term')->load($tid);
            if (is_object($taxonomy_term)) {
                if ($type == "full") {
                    return $taxonomy_term;
                } else {
                    return array('name' => $taxonomy_term->label(), 'tid' => $taxonomy_term->id());
                }
            }else{
                return array();
            }
        }else{
            return array();
        }
    }
    public function node_load_object_by_nid($nid)
    {
        return \Drupal::entityTypeManager()->getStorage('node')->load($nid);
    }
    public function taxonomy_url_alias($term_id)
    {
        return \Drupal::service('path.alias_manager')->getAliasByPath('/taxonomy/term/' . $term_id);
    }
    public function taxonomy_url($term_id)
    {
        return ('taxonomy/term/' . $term_id);
    }
    public function get_route_name_by_url($path)
    {
        $url_object = \Drupal::service('path.validator')->getUrlIfValid($path);
        return $url_object->getRouteName();
    }
    public function taxonomy_load_by_name($term_name, $vid = null)
    {
        $taxonomy_terms = taxonomy_term_load_multiple_by_name($term_name, $vid);
        $result = [];
        if (!empty($taxonomy_terms)) {
            foreach ($taxonomy_terms as $key => $taxonomy_term) {
                $result[] = array('name' => $taxonomy_term->label(), 'tid' => $taxonomy_term->id());
            }
        }
        if (count($result) == 1) {
            return array_shift($result);
        }
        return $result;
    }
    public function taxonomy_load_multi_by_vid($vid)
    {
        $connection = Database::getConnection();
        $res = $connection->select('taxonomy_term_data', 'n')
            ->fields('n', array('tid', 'vid'))
            ->condition('n.vid', $vid, '=')
            ->execute()
            ->fetchAllAssoc('tid');
        $items = [];
        foreach (array_keys($res) as $key => $tid) {
            //taxonomy_term
            $taxonomy_term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->load($tid);
            if (is_object($taxonomy_term)) {
                $items[] = array(
                    'name' => strtolower($taxonomy_term->label()),
                    'tid' => $taxonomy_term->id(),
                    'url' => $this->helper->url_taxonomy_alias($tid)
                );
            }
        }

        return $items;
    }
    public function current_url()
    {
        $url = Url::fromRoute('<current>');
        return $url->getInternalPath();
    }
    public function current_url_alias()
    {
        return Url::fromRoute('<current>');
    }
    public function get_menu_tree($menu)
    {
        $tree = \Drupal::menuTree()->load($menu, new MenuTreeParameters());
        $menu = [];
        foreach ($tree as $item) {
            $menu["url"] = $item->link->getUrlObject()->toString();
            $menu["title"] = $item->link->getTitle();
            $menu["link"] = $item->link;
        }
        return $tree;
    }
    public function str_ends_with($haystack, $needle)
    {
        return strrpos($haystack, $needle) + strlen($needle) ===
            strlen($haystack);
    }
    function str_starts_with($haystack, $needle)
    {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }
    public function get_uri_image_by_fid($fid, $style = null)
    {
        $file = \Drupal\file\Entity\File::load($fid);
        if (is_object($file)) {
            if ($style != null) {
                return \Drupal\image\Entity\ImageStyle::load($style)->buildUrl($file->getFileUri());
            } else {
                return $file->getFileUri();
            }
        } else {
            return null;
        }
    }
    public function is_login()
    {
        $user = \Drupal::currentUser();
        return !$user->isAnonymous();
    }
    public function get_block_custom_type($block_content)
    {
        if (is_object($block_content)) {
            return $block_content->bundle();
        } else {
            return null;
        }
    }
    public function node_id_by_current_node_url() {
        $path = $this->current_url();
        if ($this->str_starts_with($path, "node/")) {
            $fs_refer = explode("node/", $path);
            return $fs_refer[1];
        }
        else {
            return null;
        }
    }
    public function taxonomy_id_by_current_url() {
        $path = $this->current_url();
        if ($this->str_starts_with($path, "taxonomy/term/")) {
            $fs_refer = explode("taxonomy/term/", $path);
            return $fs_refer[1];
        }
        else {
            return null;
        }
    }
    public function  taxonomy_get_children($tid){
        $results = [];
        $children = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadChildren($tid);

        if(!empty($children)){
            $results =  array_keys($children);

            foreach ($children as $key => $child){
                $next = $this->taxonomy_get_children($key);
                if(!empty($next)){
                    $results = array_merge($results, ($next));

                }
            }

        }

        return $results ;

    }
    public function get_vocabulary_by_tid($tid){
        $term = $this->taxonomy_load_by_tid($tid,"full");
        $vid = $term->get("vid")->getValue();
        return $vid[0]['target_id'];

    }
    public  function get_vocabulary_by_taxonomy_url(){
        $tid = $this->taxonomy_id_by_current_url();
        return $this->get_vocabulary_by_tid($tid);
    }
    public function is_field_ready($entity, $field) {
        $bool = false;
        if (is_object($entity) && $entity->hasField($field)) {
            $field_value = $entity->get($field)->getValue();
            if (!empty($field_value)) {
                $bool = true;
            }
        }
        return $bool;
    }
    public function get_parameter($param = null) {
        $method = \Drupal::request()->getMethod();
        if ($param == null) {
            if($method=="GET"){
                return \Drupal::request()->query->all();}
            elseif ($method=="POST"){
                return  \Drupal::request()->request->all();
            }else{
                return null ;
            }
        }
        else {
            if($method=="GET"){
                return \Drupal::request()->query->get($param);
            } elseif ($method=="POST"){
                return  \Drupal::request()->request->get($param);
            } else{
                return null ;
            }
        }
    }
    function get_multi_query_request() {
        if ($_SERVER['QUERY_STRING']) {
            foreach (explode("&", $_SERVER['QUERY_STRING']) as $tmp_arr_param) {
                $tmp_arr_param_1 = explode("=", $tmp_arr_param);
                $filters[$tmp_arr_param_1[0]][] = $tmp_arr_param_1[1];
            }
            foreach($filters as $key => $filter){
                if(count($filter)==1){
                    $filters[$key] = array_shift($filter);
                }
            }
        }
        else {
            $filters = null;
        }

        return $filters;
    }
    //insert block custom content
    public function insert_content_block($type, $title, $fields = array()) {

        // Grab a block entity manager from EntityManager service
        $blockEntityManager = \Drupal::service('entity.manager')
            ->getStorage('block_content');
        $block_type = $this->block_custom_load_by_type($type);
        if (!empty($block_type)) {

            // Tell block entity manager to create a block of type "ad_block"
            $block = $blockEntityManager->create(array(
                'type' => $type
            ));
            // Every block should have a description, but strangely it's property
            // is not 'description' but 'info'
            // in my case, I want it to be equal to my ad_group's term name.
            $block->info = $title;
            // This is optional part, my ad_block has a field field_ad_group
            // which is a taxonomy reference to the ad_group taxonomy,
            // that way I link ad_group and ad_block together.
            foreach ($fields as $key => $field) {
                $is_exist = $this->is_field_ready($block, $key);
                if ($is_exist) {
                    $block->{$key} = $field;
                }
                else {
                    drupal_set_message('Block type:' . $type . 'do not have Field:' . $key, 'warning');
                }
            }
            //$block->field_ad_group = $entity;
            // In the end, save our new block.
            $block->save();
        }
        else {
            drupal_set_message('Block type:' . $type . ' not exist', 'error');
        }
    }
    //get module path
    public function get_module_path($module_name) {
        $module_handler = \Drupal::service('module_handler');
        return $module_handler->getModule($module_name)->getPath();
    }

    public function is_url_admin() {
        $route = \Drupal::routeMatch()->getRouteObject();
        return \Drupal::service('router.admin_context')->isAdminRoute($route);
    }

    public function paragraph_load_object($key) {
        $paragraph = \Drupal::entityTypeManager()
            ->getStorage('paragraph')
            ->load($key);
        return $paragraph;
    }
    /**
     * @param entity_type can be 'taxonomy_term' or 'node' or 'user'
     * entity_name is the name of your entity
     * @note : for get field user list $entity_type='user' and $entity_name = 'user' **/

    public function get_fields_by_entity_info($entity_type , $entity_name=null){
        $entity_type_list=array("taxonomy_term","node","user");
        if(in_array($entity_type,$entity_type_list)){
            $fields_node_config = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity_type, $entity_name);
            $fields=[];
            foreach($fields_node_config as $key=>$field){
                $fields[]=array(
                    "name"=>$key,
                    "type"=> $field->getType(),
                    "target_type"=> (isset($field->getSettings()['target_type']))? $field->getSettings()['target_type']:""
                );
            }
            return  $fields;
        }else{
            return null;
        }
    }

    public function get_type_field($entity, $field_name) {
        if (is_object($entity) && $entity->hasField($field_name)) {
            $field_type = $entity->get($field_name)->getFieldDefinition()->getType();
            return $field_type;
        }
        else {
            return null;
        }
    }
    public function get_setting_field($entity, $field_name) {
        $resulat = null;
        if (is_object($entity) && $entity->hasField($field_name)) {
            $setting_field = $entity->get($field_name)->getFieldDefinition()->getSettings(); //->getValue();
            if (isset($setting_field['target_type'])) {
                $resulat = $setting_field['target_type'];
            }
        }
        return $resulat;
    }
    public static function is_module_exist($module_name){
        return \Drupal::moduleHandler()->moduleExists($module_name);
    }
    public function taxonomy_tree($vid){
        return \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vid, 0);
    }
    public function taxonomy_children($vid,$tid){
        $tree = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vid, 0);
        $childre_term = [];
        foreach($tree as $term) {
            if (in_array($tid, $term->parents)) {
                $childre_term[] = array(
                    'name' => strtolower($term->name),
                    'tid' => $term->tid,
                    'url' => $this->url_taxonomy_alias( $term->tid)
                );

            }
        }
        return $childre_term ;
    }
  public function render_block_by_title($title,$type='all'){
      $factory = \Drupal::entityTypeManager()->getStorage("block_content")->getQuery();
      $factory->condition('info',$title);
      if($type!='all'){
      $factory->condition('type',$type);
      }
      $resultat = $factory->execute();
      $items=[];
      if(!empty($resultat)){
          foreach ($resultat as $block_id)
              $block = \Drupal::entityTypeManager()->getStorage('block_content')->load($block_id);
              $items[] = \Drupal::entityTypeManager()->getViewBuilder('block_content')->view($block);
      }
      if(count($items)==1){
          $items = array_shift($items);
      }
      return $items ;
  }


}